; Z80 CPU clock tool
; Written by (alias) Dr. D'nar
; 4 Augest 2008, revised for MathPrint 31 May 2020 thanks to Linus Tech Tips
; posting a video and generating questions about why this isn't working.  It
; hadn't been a problem for a whole decade.
; Finds the effective CPU clock of an SE/84.  Will not work on a plain TI-83+.

; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.


;====== Equates ================================================================
.nolist		; Uncomment for massive list file.
#include "ti83plus.inc"
.list

; Equates for MathPrint.  At the time of this writing, this describes everything
; that is known about about these flags.
MathPrintFlags1 .EQU 44h
MathPrintActive .EQU 5
MathPrintFlags2 .EQU 45h
MathPrintShowReminder .EQU 4
MathPrintFlags3 .EQU 46h ; It is unknown whether the OS actually uses this byte
MathPrintFlags4 .EQU 47h
; Bits 1 and 4 of this byte are related to when the OS decides to format an
; answer as a fraction 
MathPrintFlags5 .EQU 48h
MathPrintFractionMode .EQU 0 ; controls whether or not mixed number mode is used


; In theory, larger values lead to more accurate results.  Or not.  I donno.
; Update 16 Jan 2011
; Okay, so, in theory larger values lead to more accurate results.  In practice,
; the quartz crystals run at 32768 Hz, limiting the accuracy of the test to
; +/- 2 kilohertz.
numOfTicks	.equ	254; 32
numOfIterations	.equ	200;99 ; 10



;====== Program Start ==========================================================
; Header
	.org	$9D93
	.db	t2ByteTok, tAsmCmp

; Check model	
	in	a, ($02)
	bit	7, a
	jr	nz, correctModel
	ld	hl, incorrectModelMsg
	b_call(_PutS)
	b_call(_NewLine)
	ret


correctModel:
	ld	(iy + asm_Flag1), 0
	b_call(_GetBaseVer)
	cp	2
	jr	nz, noMathPrint
	ld	a, b
	cp	53
	jr	c, noMathPrint
	ld	a, (iy + MathPrintFlags1)
	ld	(iy + asm_Flag1), a
	res	MathPrintActive, (iy + MathPrintFlags1)
	bit	MathPrintActive, a
	jr	z, noMathPrint
	b_call(_ClrScrn)
	b_call(_HomeUp)
	
noMathPrint:
; Test each CPU mode
	ld	hl, startMsg
	b_call(_PutS)
	b_call(_NewLine)
	xor	a
	out	($20), a
	call	findCPUSpeed
	ld	a, 1
	out	($20), a
	call	findCPUSpeed
	ld	a, 2
	out	($20), a
	call	findCPUSpeed
	ld	a, 3
	out	($20), a
	call	findCPUSpeed
	bit	MathPrintActive, (iy + asm_Flag1)
	ret	z
	ld	a, busyPause
	ld	(indicBusy), a
waitloop:
	b_call(_GetCSC)
	or	a
	jr	z, waitloop
	set	MathPrintActive, (iy + MathPrintFlags1)
	b_call(_ClrScrn)
	b_call(_HomeUp)
	ret


findCPUSpeed:
; The following section calls the speedTest routine several times and averages
; the results.
	b_call(_OP1Set0)
	b_call(_OP1ToOP6)
	ld	hl, numOfIterations
	ld	de, 0
CPULoop:
	push	hl
	push	de
	call	speedTest
	b_call(_OP6ToOP2)
	b_call(_FPAdd)
	b_call(_OP1ToOP6)
	pop	de
	pop	hl
	dec	hl
	call	cphlde
	jr	nz, CPULoop
	
	b_call(_OP6ToOP1)
	ld	hl, numOfIterations
	b_call(_SetXXXXOP2)
	b_call(_FPDiv)
	ld	hl, 1000
	b_call(_SetXXXXOP2)
	b_call(_FPDiv)
	b_call(_FPDiv)
	ld	a, 6;13
	b_call(_FormEReal)
	ld	hl, OP3
	b_call(_PutS)
	ld	hl, MHzText
	b_call(_PutS)
	b_call(_NewLine)
	ret


speedTest:
; This uses the crystal timers to time a loop which takes a known amount of CPU 
; clock cycles.  This is then used to calculate the CPU clock speed.  If you 
; make the loop longer or shorter, remember to update the section below that
; computes the speed or you might get some wild results!  The loop should be as
; short as possible for better results.
	di
	ld	de, $FFFF
	ld	hl, 8000h
	ld	ix, 8001h
	ld	a, $44				; 32768 Hz
	out	($30), a
	xor	a
	out	($31), a
	ld	a, numOfTicks
	out	($32), a
speedLoop:
	inc	de				;  6
						; 0
	;ld	(hl), 0				; 10
	ld	(ix+0), 0			; 19
	in	a, (4)				; 11
	bit	5, a				;  8
	jr	z, speedLoop			; 12
		   				; 37 cycles
						; or 47 or 56 cycles
	xor	a
	out	($31), a
	out	($31), a
	ei
; Compute speed
;	push	hl
;	b_call(_DispHL)
;	pop	hl
	push	de
	ld	a, 1
	b_call(_SetXXOP1)
	ld	hl, 32768
	b_call(_SetXXXXOP2)
	b_call(_FPDiv)
	ld	hl, numOfTicks
	b_call(_SetXXXXOP2)
	b_call(_FPMult)
	b_call(_OP1ToOP4)
	ld	a, 56
	b_call(_SetXXOp1)
	pop	hl
	b_call(_SetXXXXOP2)
	b_call(_FPMult)
	b_call(_OP4ToOP2)
	b_call(_FPDiv)
	ret



;====== Routines ===============================================================
cphlde:
; A classic.
	or	a
	sbc	hl,de
	add	hl,de
	ret



;====== Data ===================================================================
		;1234567890123456
incorrectModelMsg:
	.db	"An SE/84 is     "
	.db	"required."
	.db	0
startMsg:
	.db	"Starting CPU    "
	.db	"speed test..."
	.db	0

MHzText:
	.db	" MHz"
	.db	0
.end
.end
